#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 8
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGERK(ATL_CINT M, ATL_CINT N, const double *X, const double *Y, 
               double *A, ATL_CINT lda)
/*
 * 40x8 unrolled r1_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N8=(N/8)*8, M40=(M/40)*40, lda8=lda*8;
   ATL_INT j;

   for (j=N8; j; j -= 8, A += lda8, Y += 8)
   {
      double *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda, *A5=A4+lda,
             *A6=A5+lda, *A7=A6+lda;
      const register double y0=Y[0], y1=Y[1], y2=Y[2], y3=Y[3], y4=Y[4],
                            y5=Y[5], y6=Y[6], y7=Y[7];
      ATL_INT i;
      prefY(Y+8+8-1);
      for (i=0; i < M40; i += 40)
      {
         const register double x0=X[i+0], x1=X[i+1], x2=X[i+2], x3=X[i+3],
                               x4=X[i+4], x5=X[i+5], x6=X[i+6], x7=X[i+7],
                               x8=X[i+8], x9=X[i+9], x10=X[i+10], x11=X[i+11],
                               x12=X[i+12], x13=X[i+13], x14=X[i+14],
                               x15=X[i+15], x16=X[i+16], x17=X[i+17],
                               x18=X[i+18], x19=X[i+19], x20=X[i+20],
                               x21=X[i+21], x22=X[i+22], x23=X[i+23],
                               x24=X[i+24], x25=X[i+25], x26=X[i+26],
                               x27=X[i+27], x28=X[i+28], x29=X[i+29],
                               x30=X[i+30], x31=X[i+31], x32=X[i+32],
                               x33=X[i+33], x34=X[i+34], x35=X[i+35],
                               x36=X[i+36], x37=X[i+37], x38=X[i+38],
                               x39=X[i+39];

         prefX(X);
         A0[i+0] += y0 * x0;
         prefA(A0);
         A1[i+0] += y1 * x0;
         prefA(A1);
         A2[i+0] += y2 * x0;
         prefA(A2);
         A3[i+0] += y3 * x0;
         prefA(A3);
         A4[i+0] += y4 * x0;
         prefA(A4);
         A5[i+0] += y5 * x0;
         prefA(A5);
         A6[i+0] += y6 * x0;
         prefA(A6);
         A7[i+0] += y7 * x0;
         prefA(A7);
         A0[i+1] += y0 * x1;
         A1[i+1] += y1 * x1;
         A2[i+1] += y2 * x1;
         A3[i+1] += y3 * x1;
         A4[i+1] += y4 * x1;
         A5[i+1] += y5 * x1;
         A6[i+1] += y6 * x1;
         A7[i+1] += y7 * x1;
         A0[i+2] += y0 * x2;
         A1[i+2] += y1 * x2;
         A2[i+2] += y2 * x2;
         A3[i+2] += y3 * x2;
         A4[i+2] += y4 * x2;
         A5[i+2] += y5 * x2;
         A6[i+2] += y6 * x2;
         A7[i+2] += y7 * x2;
         A0[i+3] += y0 * x3;
         A1[i+3] += y1 * x3;
         A2[i+3] += y2 * x3;
         A3[i+3] += y3 * x3;
         A4[i+3] += y4 * x3;
         A5[i+3] += y5 * x3;
         A6[i+3] += y6 * x3;
         A7[i+3] += y7 * x3;
         A0[i+4] += y0 * x4;
         A1[i+4] += y1 * x4;
         A2[i+4] += y2 * x4;
         A3[i+4] += y3 * x4;
         A4[i+4] += y4 * x4;
         A5[i+4] += y5 * x4;
         A6[i+4] += y6 * x4;
         A7[i+4] += y7 * x4;
         A0[i+5] += y0 * x5;
         A1[i+5] += y1 * x5;
         A2[i+5] += y2 * x5;
         A3[i+5] += y3 * x5;
         A4[i+5] += y4 * x5;
         A5[i+5] += y5 * x5;
         A6[i+5] += y6 * x5;
         A7[i+5] += y7 * x5;
         A0[i+6] += y0 * x6;
         A1[i+6] += y1 * x6;
         A2[i+6] += y2 * x6;
         A3[i+6] += y3 * x6;
         A4[i+6] += y4 * x6;
         A5[i+6] += y5 * x6;
         A6[i+6] += y6 * x6;
         A7[i+6] += y7 * x6;
         A0[i+7] += y0 * x7;
         A1[i+7] += y1 * x7;
         A2[i+7] += y2 * x7;
         A3[i+7] += y3 * x7;
         A4[i+7] += y4 * x7;
         A5[i+7] += y5 * x7;
         A6[i+7] += y6 * x7;
         A7[i+7] += y7 * x7;
         A0[i+8] += y0 * x8;
         prefA(A0);
         A1[i+8] += y1 * x8;
         prefA(A1);
         A2[i+8] += y2 * x8;
         prefA(A2);
         A3[i+8] += y3 * x8;
         prefA(A3);
         A4[i+8] += y4 * x8;
         prefA(A4);
         A5[i+8] += y5 * x8;
         prefA(A5);
         A6[i+8] += y6 * x8;
         prefA(A6);
         A7[i+8] += y7 * x8;
         prefA(A7);
         A0[i+9] += y0 * x9;
         A1[i+9] += y1 * x9;
         A2[i+9] += y2 * x9;
         A3[i+9] += y3 * x9;
         A4[i+9] += y4 * x9;
         A5[i+9] += y5 * x9;
         A6[i+9] += y6 * x9;
         A7[i+9] += y7 * x9;
         A0[i+10] += y0 * x10;
         A1[i+10] += y1 * x10;
         A2[i+10] += y2 * x10;
         A3[i+10] += y3 * x10;
         A4[i+10] += y4 * x10;
         A5[i+10] += y5 * x10;
         A6[i+10] += y6 * x10;
         A7[i+10] += y7 * x10;
         A0[i+11] += y0 * x11;
         A1[i+11] += y1 * x11;
         A2[i+11] += y2 * x11;
         A3[i+11] += y3 * x11;
         A4[i+11] += y4 * x11;
         A5[i+11] += y5 * x11;
         A6[i+11] += y6 * x11;
         A7[i+11] += y7 * x11;
         A0[i+12] += y0 * x12;
         A1[i+12] += y1 * x12;
         A2[i+12] += y2 * x12;
         A3[i+12] += y3 * x12;
         A4[i+12] += y4 * x12;
         A5[i+12] += y5 * x12;
         A6[i+12] += y6 * x12;
         A7[i+12] += y7 * x12;
         A0[i+13] += y0 * x13;
         A1[i+13] += y1 * x13;
         A2[i+13] += y2 * x13;
         A3[i+13] += y3 * x13;
         A4[i+13] += y4 * x13;
         A5[i+13] += y5 * x13;
         A6[i+13] += y6 * x13;
         A7[i+13] += y7 * x13;
         A0[i+14] += y0 * x14;
         A1[i+14] += y1 * x14;
         A2[i+14] += y2 * x14;
         A3[i+14] += y3 * x14;
         A4[i+14] += y4 * x14;
         A5[i+14] += y5 * x14;
         A6[i+14] += y6 * x14;
         A7[i+14] += y7 * x14;
         A0[i+15] += y0 * x15;
         A1[i+15] += y1 * x15;
         A2[i+15] += y2 * x15;
         A3[i+15] += y3 * x15;
         A4[i+15] += y4 * x15;
         A5[i+15] += y5 * x15;
         A6[i+15] += y6 * x15;
         A7[i+15] += y7 * x15;
         A0[i+16] += y0 * x16;
         prefA(A0);
         A1[i+16] += y1 * x16;
         prefA(A1);
         A2[i+16] += y2 * x16;
         prefA(A2);
         A3[i+16] += y3 * x16;
         prefA(A3);
         A4[i+16] += y4 * x16;
         prefA(A4);
         A5[i+16] += y5 * x16;
         prefA(A5);
         A6[i+16] += y6 * x16;
         prefA(A6);
         A7[i+16] += y7 * x16;
         prefA(A7);
         A0[i+17] += y0 * x17;
         A1[i+17] += y1 * x17;
         A2[i+17] += y2 * x17;
         A3[i+17] += y3 * x17;
         A4[i+17] += y4 * x17;
         A5[i+17] += y5 * x17;
         A6[i+17] += y6 * x17;
         A7[i+17] += y7 * x17;
         A0[i+18] += y0 * x18;
         A1[i+18] += y1 * x18;
         A2[i+18] += y2 * x18;
         A3[i+18] += y3 * x18;
         A4[i+18] += y4 * x18;
         A5[i+18] += y5 * x18;
         A6[i+18] += y6 * x18;
         A7[i+18] += y7 * x18;
         A0[i+19] += y0 * x19;
         A1[i+19] += y1 * x19;
         A2[i+19] += y2 * x19;
         A3[i+19] += y3 * x19;
         A4[i+19] += y4 * x19;
         A5[i+19] += y5 * x19;
         A6[i+19] += y6 * x19;
         A7[i+19] += y7 * x19;
         A0[i+20] += y0 * x20;
         A1[i+20] += y1 * x20;
         A2[i+20] += y2 * x20;
         A3[i+20] += y3 * x20;
         A4[i+20] += y4 * x20;
         A5[i+20] += y5 * x20;
         A6[i+20] += y6 * x20;
         A7[i+20] += y7 * x20;
         A0[i+21] += y0 * x21;
         A1[i+21] += y1 * x21;
         A2[i+21] += y2 * x21;
         A3[i+21] += y3 * x21;
         A4[i+21] += y4 * x21;
         A5[i+21] += y5 * x21;
         A6[i+21] += y6 * x21;
         A7[i+21] += y7 * x21;
         A0[i+22] += y0 * x22;
         A1[i+22] += y1 * x22;
         A2[i+22] += y2 * x22;
         A3[i+22] += y3 * x22;
         A4[i+22] += y4 * x22;
         A5[i+22] += y5 * x22;
         A6[i+22] += y6 * x22;
         A7[i+22] += y7 * x22;
         A0[i+23] += y0 * x23;
         A1[i+23] += y1 * x23;
         A2[i+23] += y2 * x23;
         A3[i+23] += y3 * x23;
         A4[i+23] += y4 * x23;
         A5[i+23] += y5 * x23;
         A6[i+23] += y6 * x23;
         A7[i+23] += y7 * x23;
         A0[i+24] += y0 * x24;
         prefA(A0);
         A1[i+24] += y1 * x24;
         prefA(A1);
         A2[i+24] += y2 * x24;
         prefA(A2);
         A3[i+24] += y3 * x24;
         prefA(A3);
         A4[i+24] += y4 * x24;
         prefA(A4);
         A5[i+24] += y5 * x24;
         prefA(A5);
         A6[i+24] += y6 * x24;
         prefA(A6);
         A7[i+24] += y7 * x24;
         prefA(A7);
         A0[i+25] += y0 * x25;
         A1[i+25] += y1 * x25;
         A2[i+25] += y2 * x25;
         A3[i+25] += y3 * x25;
         A4[i+25] += y4 * x25;
         A5[i+25] += y5 * x25;
         A6[i+25] += y6 * x25;
         A7[i+25] += y7 * x25;
         A0[i+26] += y0 * x26;
         A1[i+26] += y1 * x26;
         A2[i+26] += y2 * x26;
         A3[i+26] += y3 * x26;
         A4[i+26] += y4 * x26;
         A5[i+26] += y5 * x26;
         A6[i+26] += y6 * x26;
         A7[i+26] += y7 * x26;
         A0[i+27] += y0 * x27;
         A1[i+27] += y1 * x27;
         A2[i+27] += y2 * x27;
         A3[i+27] += y3 * x27;
         A4[i+27] += y4 * x27;
         A5[i+27] += y5 * x27;
         A6[i+27] += y6 * x27;
         A7[i+27] += y7 * x27;
         A0[i+28] += y0 * x28;
         A1[i+28] += y1 * x28;
         A2[i+28] += y2 * x28;
         A3[i+28] += y3 * x28;
         A4[i+28] += y4 * x28;
         A5[i+28] += y5 * x28;
         A6[i+28] += y6 * x28;
         A7[i+28] += y7 * x28;
         A0[i+29] += y0 * x29;
         A1[i+29] += y1 * x29;
         A2[i+29] += y2 * x29;
         A3[i+29] += y3 * x29;
         A4[i+29] += y4 * x29;
         A5[i+29] += y5 * x29;
         A6[i+29] += y6 * x29;
         A7[i+29] += y7 * x29;
         A0[i+30] += y0 * x30;
         A1[i+30] += y1 * x30;
         A2[i+30] += y2 * x30;
         A3[i+30] += y3 * x30;
         A4[i+30] += y4 * x30;
         A5[i+30] += y5 * x30;
         A6[i+30] += y6 * x30;
         A7[i+30] += y7 * x30;
         A0[i+31] += y0 * x31;
         A1[i+31] += y1 * x31;
         A2[i+31] += y2 * x31;
         A3[i+31] += y3 * x31;
         A4[i+31] += y4 * x31;
         A5[i+31] += y5 * x31;
         A6[i+31] += y6 * x31;
         A7[i+31] += y7 * x31;
         A0[i+32] += y0 * x32;
         prefA(A0);
         A1[i+32] += y1 * x32;
         prefA(A1);
         A2[i+32] += y2 * x32;
         prefA(A2);
         A3[i+32] += y3 * x32;
         prefA(A3);
         A4[i+32] += y4 * x32;
         prefA(A4);
         A5[i+32] += y5 * x32;
         prefA(A5);
         A6[i+32] += y6 * x32;
         prefA(A6);
         A7[i+32] += y7 * x32;
         prefA(A7);
         A0[i+33] += y0 * x33;
         A1[i+33] += y1 * x33;
         A2[i+33] += y2 * x33;
         A3[i+33] += y3 * x33;
         A4[i+33] += y4 * x33;
         A5[i+33] += y5 * x33;
         A6[i+33] += y6 * x33;
         A7[i+33] += y7 * x33;
         A0[i+34] += y0 * x34;
         A1[i+34] += y1 * x34;
         A2[i+34] += y2 * x34;
         A3[i+34] += y3 * x34;
         A4[i+34] += y4 * x34;
         A5[i+34] += y5 * x34;
         A6[i+34] += y6 * x34;
         A7[i+34] += y7 * x34;
         A0[i+35] += y0 * x35;
         A1[i+35] += y1 * x35;
         A2[i+35] += y2 * x35;
         A3[i+35] += y3 * x35;
         A4[i+35] += y4 * x35;
         A5[i+35] += y5 * x35;
         A6[i+35] += y6 * x35;
         A7[i+35] += y7 * x35;
         A0[i+36] += y0 * x36;
         A1[i+36] += y1 * x36;
         A2[i+36] += y2 * x36;
         A3[i+36] += y3 * x36;
         A4[i+36] += y4 * x36;
         A5[i+36] += y5 * x36;
         A6[i+36] += y6 * x36;
         A7[i+36] += y7 * x36;
         A0[i+37] += y0 * x37;
         A1[i+37] += y1 * x37;
         A2[i+37] += y2 * x37;
         A3[i+37] += y3 * x37;
         A4[i+37] += y4 * x37;
         A5[i+37] += y5 * x37;
         A6[i+37] += y6 * x37;
         A7[i+37] += y7 * x37;
         A0[i+38] += y0 * x38;
         A1[i+38] += y1 * x38;
         A2[i+38] += y2 * x38;
         A3[i+38] += y3 * x38;
         A4[i+38] += y4 * x38;
         A5[i+38] += y5 * x38;
         A6[i+38] += y6 * x38;
         A7[i+38] += y7 * x38;
         A0[i+39] += y0 * x39;
         A1[i+39] += y1 * x39;
         A2[i+39] += y2 * x39;
         A3[i+39] += y3 * x39;
         A4[i+39] += y4 * x39;
         A5[i+39] += y5 * x39;
         A6[i+39] += y6 * x39;
         A7[i+39] += y7 * x39;
      }
      for (i=M40; i < M; i++)
      {
         const register double x0 = X[i];
         A0[i] += y0 * x0;
         A1[i] += y1 * x0;
         A2[i] += y2 * x0;
         A3[i] += y3 * x0;
         A4[i] += y4 * x0;
         A5[i] += y5 * x0;
         A6[i] += y6 * x0;
         A7[i] += y7 * x0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N8; j; j--, A += lda, Y++)
   {
      double *A0=A;
      const register double y0=Y[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      for (i=0; i < M40; i += 40)
      {
         const register double x0=X[i+0], x1=X[i+1], x2=X[i+2], x3=X[i+3],
                               x4=X[i+4], x5=X[i+5], x6=X[i+6], x7=X[i+7],
                               x8=X[i+8], x9=X[i+9], x10=X[i+10], x11=X[i+11],
                               x12=X[i+12], x13=X[i+13], x14=X[i+14],
                               x15=X[i+15], x16=X[i+16], x17=X[i+17],
                               x18=X[i+18], x19=X[i+19], x20=X[i+20],
                               x21=X[i+21], x22=X[i+22], x23=X[i+23],
                               x24=X[i+24], x25=X[i+25], x26=X[i+26],
                               x27=X[i+27], x28=X[i+28], x29=X[i+29],
                               x30=X[i+30], x31=X[i+31], x32=X[i+32],
                               x33=X[i+33], x34=X[i+34], x35=X[i+35],
                               x36=X[i+36], x37=X[i+37], x38=X[i+38],
                               x39=X[i+39];

         prefX(X);
         A0[i+0] += y0 * x0;
         prefA(A0);
         A0[i+1] += y0 * x1;
         A0[i+2] += y0 * x2;
         A0[i+3] += y0 * x3;
         A0[i+4] += y0 * x4;
         A0[i+5] += y0 * x5;
         A0[i+6] += y0 * x6;
         A0[i+7] += y0 * x7;
         A0[i+8] += y0 * x8;
         prefA(A0);
         A0[i+9] += y0 * x9;
         A0[i+10] += y0 * x10;
         A0[i+11] += y0 * x11;
         A0[i+12] += y0 * x12;
         A0[i+13] += y0 * x13;
         A0[i+14] += y0 * x14;
         A0[i+15] += y0 * x15;
         A0[i+16] += y0 * x16;
         prefA(A0);
         A0[i+17] += y0 * x17;
         A0[i+18] += y0 * x18;
         A0[i+19] += y0 * x19;
         A0[i+20] += y0 * x20;
         A0[i+21] += y0 * x21;
         A0[i+22] += y0 * x22;
         A0[i+23] += y0 * x23;
         A0[i+24] += y0 * x24;
         prefA(A0);
         A0[i+25] += y0 * x25;
         A0[i+26] += y0 * x26;
         A0[i+27] += y0 * x27;
         A0[i+28] += y0 * x28;
         A0[i+29] += y0 * x29;
         A0[i+30] += y0 * x30;
         A0[i+31] += y0 * x31;
         A0[i+32] += y0 * x32;
         prefA(A0);
         A0[i+33] += y0 * x33;
         A0[i+34] += y0 * x34;
         A0[i+35] += y0 * x35;
         A0[i+36] += y0 * x36;
         A0[i+37] += y0 * x37;
         A0[i+38] += y0 * x38;
         A0[i+39] += y0 * x39;
      }
      for (i=M40; i < M; i++)
      {
         const register double x0 = X[i];
         A0[i] += y0 * x0;
      }
   }
}
